"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hook = void 0;
const nanoid_1 = require("nanoid");
const utils_1 = require("./utils");
class Hook {
    constructor(register, alarms, alarmPathways) {
        this.register = register;
        this.alarms = alarms;
        this.alarmPathways = alarmPathways;
    }
    run(values, typedValues) {
        return __awaiter(this, void 0, void 0, function* () {
            const compiledAlarms = this.alarms.map((x) => {
                const runtimeId = (0, nanoid_1.nanoid)();
                const hook = (0, utils_1.makeHook)(x.script || '', this.alarmPathways, (message, level, sticky) => __awaiter(this, void 0, void 0, function* () {
                    return yield this.raiseAlarm(x.id, message, level, sticky);
                }), (message, pathway) => __awaiter(this, void 0, void 0, function* () {
                    return yield this.sendNotification(message, pathway);
                }));
                return hook;
            });
            yield Promise.all(compiledAlarms.map((alm) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                yield ((_a = alm.handler) === null || _a === void 0 ? void 0 : _a.call(alm, values, typedValues));
            })));
        });
    }
    raiseAlarm(causeId, message, level, sticky) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            let shouldAlarm = true;
            if (sticky) {
                const lastAlarm = yield ((_b = (_a = this.register).getLast) === null || _b === void 0 ? void 0 : _b.call(_a, message, causeId, level ? utils_1.ALARM_LEVEL[level] : undefined));
                // const lastAlarm = await this.prisma?.alarm.findFirst({
                //     where: {
                //         deviceId: deviceId,
                //         message,
                //         severity:level ? ALARM_LEVEL[level]: undefined
                //     }
                // })
                if (lastAlarm && !(lastAlarm === null || lastAlarm === void 0 ? void 0 : lastAlarm.ack)) {
                    shouldAlarm = false;
                }
            }
            console.log({ shouldAlarm });
            if (shouldAlarm) {
                yield ((_d = (_c = this.register).create) === null || _d === void 0 ? void 0 : _d.call(_c, message, causeId, level ? utils_1.ALARM_LEVEL[level] : undefined));
                // await this.prisma?.alarm.create({
                //     data: {
                //         id: nanoid(),
                //         message,
                //         severity: level ? ALARM_LEVEL[level] : undefined,
                //         device: {
                //             connect: {id: deviceId}
                //         },
                //         cause: {
                //             connect: {
                //                 id: causeId
                //             }
                //         }
                //     }
                // })
            }
            else {
                return false;
            }
            return true;
        });
    }
    sendNotification(message, pathway) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const pathwayObj = (_b = (_a = this.alarmPathways) === null || _a === void 0 ? void 0 : _a.find((a) => a.name == pathway)) === null || _b === void 0 ? void 0 : _b.script;
            if (!pathwayObj)
                return null;
            const { sendNotification } = (0, utils_1.makeNotification)(pathwayObj);
            console.log("Sending notification");
            return yield (sendNotification === null || sendNotification === void 0 ? void 0 : sendNotification(message));
        });
    }
}
exports.Hook = Hook;
